#!/local/gauche/bin/gosh

(use gl)
(use gl.glut)
(use srfi-1)
(use srfi-13)
(use gauche.uvector)

(define xangle 0)
(define yangle 0)
(define timer-interval 100)
(define trail-colors '())
(define frames '())
(define current-frame '())
(define paint-list '())

(define (make-paint-list cf pl count)
  (if (null? cf) (set! cf frames))
  (if (= count 0)
    pl
    (cons cf (make-paint-list (cdr cf) pl (- count 1)))
  )
)

(define (timer x)
  (set! current-frame (if (null? current-frame) frames (cdr current-frame)))
  (if (null? current-frame) (set! current-frame frames))
  (disp)
  (glut-timer-func timer-interval timer 0)
)

(define (disp)
  (gl-clear GL_COLOR_BUFFER_BIT)
  (gl-rotate xangle 10.0 0.0 0.0)
  (gl-rotate yangle 0.0 10.0 0.0)
  (set! xangle 0.0)
  (set! yangle 0.0)
  (gl-color '#f32(0.5 0.5 0.5))
  (glut-wire-cube 10.0)
  (for-each
    (lambda (f c)
      (gl-color c)
      (gl-begin* GL_POINTS
        (for-each
          (lambda (p) (gl-vertex p))
          (car f)
        )
      )
    )
    (make-paint-list current-frame '() (length trail-colors))
    (reverse trail-colors)
  )
  (gl-flush)
)

(define (init)
  (gl-clear-color 0.0 0.0 0.0 0.0)
  (gl-matrix-mode GL_PROJECTION)
  (gl-load-identity)
  (gl-ortho -10.0 10.0 -10.0 10.0 -10.0 10.0)
)

(define (keyboard key x y)
  (cond
   ((= key 27) (exit 0))
   ((= key 119)
     (set! xangle (+ 1 xangle))
     (if (> xangle 359) (set! xangle 0.0))
   )
   ((= key 115)
     (set! xangle (- xangle 1))
     (if (< xangle 0.0) (set! xangle 359.0))
   )
   ((= key 100)
     (set! yangle (+ 1 yangle))
     (if (> yangle 359) (set! yangle 0.0))
   )
   ((= key 97)
     (set! yangle (- yangle 1))
     (if (< yangle 0.0) (set! yangle 359.0))
   )
   ((= key 43)
     (set! timer-interval (+ 10 timer-interval))
     (if (> timer-interval 10000) (set! timer-interval 10000))
   )
   ((= key 45)
     (set! timer-interval (- timer-interval 10))
     (if (< timer-interval 1) (set! timer-interval 1))
   )
  )
)

(define (get-plist coord-list frame-list)
  (if (null? coord-list)
    frame-list
    (let ((v (make-f32vector 3)))
      (f32vector-set! v 0 (string->number (first coord-list)))
      (f32vector-set! v 1 (string->number (second coord-list)))
      (f32vector-set! v 2 (string->number (third coord-list)))
      (get-plist (cdddr coord-list) (cons v frame-list))
    )
  )
)

(define (read-traj)
  (let ((flist '()))
    (port-for-each
      (lambda (l)
        (let ((c (string-tokenize l)))
          (set! flist (cons (get-plist (cdr c) '()) flist))
        )
      )
      read-line
    )
    flist
  )
)

(define (generate-trail-colors count trail-length tcl)
  (if (= count trail-length)
    tcl
    (let ((cv (make-f32vector 3 (/ count trail-length))))
      (generate-trail-colors (+ count 1) trail-length (cons cv tcl))
    )
  )
)

(define (main args)
  (set! frames (read-traj))
  (set! current-frame frames)
  (set! trail-colors (generate-trail-colors 0 30 '()))
  (glut-init args)
  (glut-init-display-mode (logior GLUT_SINGLE GLUT_RGB))
  (glut-init-window-size 250 250)
  (glut-init-window-position 100 100)
  (glut-create-window "traj")
  (init)
  (glut-display-func disp)
  (glut-keyboard-func keyboard)
  (glut-timer-func timer-interval timer 0)
  (glut-main-loop)
)
